# ************************************************************************
# * Copyright (c) infrae in 2002
# * For license details please see the license.txt file. 
# *
# * Project: Silva XML to PDF Converter
# *
# * Creation Date: May 2002
# *
# * Authors: Stephan Richter
# ************************************************************************
"""
This module contains all style and markup settings. Feel free to make changes
as you need them. After you saved the changes, you can run the following again
in order to re-create the PDF document:

run <XML File Path> [<PDF File Path>]

$Id: SilvaStyles.py,v 1.1.1.1 2002/11/13 21:29:36 itamar Exp $
"""
from reportlab.platypus.tables import TableStyle
from reportlab.lib import colors
from reportlab.lib.styles import *
from reportlab.lib.pagesizes import *
from reportlab.lib.units import inch, cm
from reportlab.lib.enums import TA_LEFT, TA_RIGHT, TA_CENTER, TA_JUSTIFY


# ** Generic Page Layout Information **

PAGESIZE          = A5
PAGEBORDER_TOP    = 1.5*cm
PAGEBORDER_BOTTOM = 1.8*cm
PAGEBORDER_LEFT   = 1.5*cm
PAGEBORDER_RIGHT  = 1.5*cm

PAGE_WIDTH = PAGESIZE[0] - PAGEBORDER_RIGHT - PAGEBORDER_LEFT
PAGE_HEIGHT = PAGESIZE[1] - PAGEBORDER_TOP - PAGEBORDER_BOTTOM

# This specifies the space that needs to be left on a page so a header will be
# put on the old page. This way we avoid having headers on the bottom with the
# text starting on a new page.
SPACE_LEFT_BEFORE_HEADER = 2*cm

# This information is displayed on the bottom of the page
FOOTER_TEXT = 'Business Administration, Erasmus University Rotterdam'
PAGENUMBER_STRING = '%i'
FOOTER_LINE = 0.5

FOOTER_FONTNAME = 'Times-Roman'
FOOTER_FONTSIZE = 9

# Note: These symbols are displayed in ZapfDingbats
LIST_TYPES = {'bullet' : 'l',
              'circle' : 'm',
              'square' : 'o',
              'dash'   : 'u',
              'none'   : ''}


stylesheet = StyleSheet1()

# ** Paragraph Styles **

stylesheet.add(ParagraphStyle(name='Normal',
                              fontName='Times-Roman',
                              fontSize=9,
                              leading=10)
               )

stylesheet.add(ParagraphStyle(name='ParagraphBeforeList',
                              fontName='Times-Roman',
                              beforeSpace=0,
                              afterSpace=0,
                              fontSize=9,
                              leading=10)
               )

stylesheet.add(ParagraphStyle(name='BodyText',
                              parent=stylesheet['Normal'],
                              spaceBefore=0,
                              spaceAfter=3)
               )


# ** List and List Item Styles **

stylesheet.add(ParagraphStyle(name='ListItem',
                              parent=stylesheet['Normal'],
                              spaceBefore=0,
                              spaceAfter=0,
                              leftIndent=30,
                              bulletIndent=20,
                              bulletFontName="ZapfDingbats",
                              bulletFontSize=6)
               )

initialIndent = 0

stylesheet.add(ParagraphStyle(name='ListItemLevel1',
                              parent=stylesheet['ListItem'],
                              leftIndent=initialIndent+10,
                              bulletIndent=initialIndent,
                              )
               )


for level in range(1, 7):
    stylesheet.add(ParagraphStyle(name='ListItemLevel%i' %(level+1),
                                  parent=stylesheet['ListItem'],
                                  leftIndent=initialIndent+10+10*level,
                                  bulletIndent=initialIndent+10*level)
                   )


# ** Header Styles **

stylesheet.add(ParagraphStyle(name='Header',
                              parent=stylesheet['Normal'],
                              spaceBefore=0,
                              spaceAfter=4,
                              indent=40, 
                              fontName='Helvetica',
                              fontSize=14)
               )

stylesheet.add(ParagraphStyle(name='Header1',
                              parent=stylesheet['Header'],
                              spaceBefore=0,
                              spaceAfter=5,
                              leading=16,
                              fontSize=14)
               )

stylesheet.add(ParagraphStyle(name='Header2',
                              parent=stylesheet['Header'],
                              spaceBefore=0,
                              spaceAfter=3,
                              leading=14,
                              fontSize=12)
               )

stylesheet.add(ParagraphStyle(name='Header3',
                              parent=stylesheet['Header'],
                              spaceBefore=0,
                              spaceAfter=2,
                              leading=12,
                              fontName='Helvetica-Bold',
                              fontSize=10)
               )

stylesheet.add(ParagraphStyle(name='Header4',
                              parent=stylesheet['Header'],
                              spaceBefore=0,
                              spaceAfter=2,
                              fontName='Helvetica-Bold',
                              leading=14,
                              fontSize=12)
               )

stylesheet.add(ParagraphStyle(name='Heading',
                              parent=stylesheet['Header'],
                              spaceBefore=5,
                              spaceAfter=3,
                              leading=12,
                              fontName='Helvetica-Bold',
                              fontSize=10)
               )


stylesheet.add(ParagraphStyle(name='CourseHeader',
                              parent=stylesheet['Normal'],
                              spaceBefore=0,
                              spaceAfter=3,
                              leading=13,
                              fontName='Helvetica-Bold',
                              fontSize=12)
               )

stylesheet.add(ParagraphStyle(name='CourseGoalAndContentHeader',
                              parent=stylesheet['Normal'],
                              spaceBefore=4,
                              spaceAfter=3,
                              fontName='Helvetica-Bold',
                              fontSize=11)
               )


# ** Front-Page-specific styles and settings **

stylesheet.add(ParagraphStyle(name='PublicationHeader',
                              parent=stylesheet['Header1'],
                              alignment = TA_CENTER,
                              fontName='Helvetica-Bold',
                              fontSize=22)
               )

stylesheet.add(ParagraphStyle(name='SmallPublicationHeader',
                              parent=stylesheet['PublicationHeader'],
                              fontSize=17)
               )

SMALL_HEADER_TEXT = 'Study Guide 2002/2003'
LOGO_PATH = '/home/srichter/infrae/logo.png'
LOGO_SIZE = (5*cm, 11*cm)


# ** Setup of the various supported Table Styles **

# NOTE: It is assumed that all textual data are wrapped by other flowables,
#       usually Paragraphs.

# --> Plain Table Style
plaintable = TableStyle(
    [ ('LEFTPADDING', (0, 0), (-1, -1), 1),
      ('RIGHTPADDING', (0, 0), (-1, -1), 1),
      ('TOPPADDING', (0, 0), (-1, -1), 0),
      ('BOTTOMPADDING', (0, 0), (-1, -1), 0),
    ])
plaintable.name='PlainTable'
stylesheet.add(plaintable)

stylesheet.add(ParagraphStyle(name='PlainTableHeader',
                              parent=stylesheet['Normal'],
                              spaceBefore=0,
                              spaceAfter=0,
                              leading=10,
                              fontName='Helvetica-Bold',
                              fontSize=9)
               )

stylesheet.add(ParagraphStyle(name='PlainTableField',
                              parent=stylesheet['Normal'],
                              spaceAfter=0)
               )

# --> Listing Table Style
listingtable = TableStyle(
    [ ('LEFTPADDING', (0, 0), (-1, -1), 1),
      ('RIGHTPADDING', (0, 0), (-1, -1), 1),
      ('TOPPADDING', (0, 0), (-1, -1), 0),
      ('BOTTOMPADDING', (0, 0), (-1, -1), 0),
      ('LINEBELOW', (0, 0), (-1, 0), 0.5, colors.black),
    ])
listingtable.name='ListingTable'
stylesheet.add(listingtable)

stylesheet.add(ParagraphStyle(name='ListingTableHeader',
                              parent=stylesheet['Normal'],
                              spaceBefore=0,
                              spaceAfter=0,
                              leading=10.5,
                              fontName='Helvetica-Bold',
                              fontSize=8)
               )

stylesheet.add(ParagraphStyle(name='ListingTableField',
                              parent=stylesheet['Normal'],
                              spaceAfter=0,
                              leading=8.7,
                              fontSize=8)
               )


# --> Grid Table Style
gridtable = TableStyle(
    [ ('LEFTPADDING', (0, 0), (-1, -1), 3),
      ('RIGHTPADDING', (0, 0), (-1, -1), 3),
      ('TOPPADDING', (0, 0), (-1, -1), 2),
      ('BOTTOMPADDING', (0, 0), (-1, -1), 2),
      ('GRID', (0, 0), (-1, -1), 0.5, colors.black),
    ])
gridtable.name='GridTable'
stylesheet.add(gridtable)

stylesheet.add(ParagraphStyle(name='GridTableHeader',
                              parent=stylesheet['Normal'],
                              spaceBefore=0,
                              spaceAfter=0,
                              leading=10,
                              fontName='Helvetica-Bold',
                              fontSize=9)
               )

stylesheet.add(ParagraphStyle(name='GridTableField',
                              parent=stylesheet['Normal'],
                              spaceAfter=0,
                              leading=10,
                              fontSize=9)
               )


# --> Data Grid Table Style
datagridtable = TableStyle(
    [ ('LEFTPADDING', (0, 0), (-1, -1), 3),
      ('RIGHTPADDING', (0, 0), (-1, -1), 3),
      ('TOPPADDING', (0, 0), (-1, -1), 2),
      ('BOTTOMPADDING', (0, 0), (-1, -1), 2),
      ('GRID', (0, 1), (-1, -1), 0.5, colors.black),
    ])
datagridtable.name='DataGridTable'
stylesheet.add(datagridtable)

stylesheet.add(ParagraphStyle(name='DataGridTableHeader',
                              parent=stylesheet['Normal'],
                              spaceBefore=0,
                              spaceAfter=0,
                              leading=10,
                              fontName='Helvetica-Bold',
                              fontSize=9)
               )

stylesheet.add(ParagraphStyle(name='DataGridTableField',
                              parent=stylesheet['Normal'],
                              spaceAfter=0,
                              leading=10,
                              fontSize=9)
               )


# ** Setup of Course Details Table Style ** 
ts = TableStyle(
    [ # setup the descriptor
      ('FONT', (0, 0), (0, -1), 'Helvetica-Bold', 9, 11),
      ('VALIGN', (0, 0), (-1, -1), 'TOP'),
      # setup the information 
      ('FONT', (1, 0), (1, -1), 'Times-Roman', 9, 11),
      # global settings
      ('LEFTPADDING', (0, 0), (-1, -1), 0),
      ('RIGHTPADDING', (0, 0), (-1, -1), 0),
      ('TOPPADDING', (0, 0), (-1, -1), 0),
      ('BOTTOMPADDING', (0, 0), (-1, -1), 0),
    ])
ts.name='CourseDetailsTable'
stylesheet.add(ts)

COURSE_CONTENT_TITLE = 'Global Description of course content'
COURSE_GOALS_TITLE = 'Goal(s) of the course'

# Here we specify what type of description header we use for each bt of course
# information.
COURSE_INFO_DESC = {'teaching_staff'   : 'Teaching Staff',
                    'coordinator'      : 'Coordinator',
                    'consulting_hour'  : 'Consulting Hour',
                    'term'             : 'Term',
                    'load'             : 'Load',
                    'educational_form' : 'Educational Form',
                    'examination'      : 'Examination',
                    'literature'       : 'Literature'}


# ** Table of Contents Styles **
delta = 1*cm
epsilon = 0.5*cm

stylesheet.add(ParagraphStyle(name='TOCLevel0',
                              fontName='Times-Roman',
                              fontSize=9,
                              leading=9,
                              firstLineIndent = -epsilon,
                              leftIndent = 0*delta + epsilon))

for level in range(1, 7):
    stylesheet.add(ParagraphStyle(name='TOCLevel%i' %level,
                                  parent = stylesheet['TOCLevel%i' %(level-1)],
                                  leading=11,
                                  firstLineIndent = -epsilon,
                                  leftIndent = level*delta + epsilon))


TOCTable = \
    TableStyle([('VALIGN', (0,0), (-1,-1), 'TOP'),
                ('LEFTPADDING', (0,0), (-1,-1), 0),
                ('RIGHTPADDING', (0,0), (-1,-1), 0),
                ('TOPPADDING', (0,0), (-1,-1), 0),
                ('BOTTOMPADDING', (0,0), (-1,-1), 2),
                ])
TOCTable.name='TOCTable'
stylesheet.add(TOCTable)

TOC_TITLE = 'Table of Contents'


# ** Index Styles **
stylesheet.add(ParagraphStyle(name='IndexText',
                              parent=stylesheet['Normal'],
                              spaceBefore=0,
                              spaceAfter=0)
               )

IndexTable = \
    TableStyle([('VALIGN', (0,0), (-1,-1), 'TOP'),
                ('LEFTPADDING', (0,0), (-1,-1), 0),
                ('RIGHTPADDING', (0,0), (-1,-1), 0),
                ('TOPPADDING', (0,0), (-1,-1), 0),
                ('BOTTOMPADDING', (0,0), (-1,-1), 1),
                ])
IndexTable.name='IndexTable'
stylesheet.add(IndexTable)

PERSON_INDEX_TITLE = 'Person Index'
INDEX_TITLE = 'Index'


# ** Colophon Styles **
stylesheet.add(ParagraphStyle(name='ColophonHeader',
                              parent=stylesheet['Normal'],
                              spaceBefore=0,
                              spaceAfter=6,
                              fontName='Helvetica-Bold',
                              fontSize=9))

ColophonTable = \
    TableStyle([('VALIGN', (0,0), (-1,-1), 'TOP'),
                ('FONT', (0, 0), (-1, -1), 'Times-Roman', 9, 10),
                ('LEFTPADDING', (0,0), (-1,-1), 0),
                ('RIGHTPADDING', (0,0), (-1,-1), 0),
                ('TOPPADDING', (0,0), (-1,-1), 0),
                ('BOTTOMPADDING', (0,0), (-1,-1), 0),
                ])
ColophonTable.name='ColophonTable'
stylesheet.add(ColophonTable)

COLOPHON_DATA = [
    ['ISBN-number', '905677204x'],
    ['Edition',     'Business Administration, Erasmus University '+
     'Rotterdam, August 2002'],
    ['Editors',     'Mw. drs. I. van Steennis'],
    ['',            'Student Affairs (Students\' Charter ' +
     '2002-2003, Institution Part)'],
    ['Co-editors',  'Mw. drs. T.N. den Hartog'],
    ['',            'Drs. A. Meijdam'],
    ['Printing',    'JB&A Grafische Communicatie, Delft - '+
     'Wateringen']]

