# ********************************i****************************************
# * Copyright (c) infrae in 2002
# * For license details please see the license.txt file. 
# *
# * Project: DOC to Silva XML Converter
# *
# * Creation Date: June 2002
# *
# * Authors: Stephan Richter, Christian Zagrodnick
# ************************************************************************
"""

$Id: P.py,v 1.15 2003/02/03 21:55:51 itamar Exp $
"""

import re

from util import replaceMap, stripControlChars
from silva2word.WordConstants import *
from word2silva.SilvaObject import SilvaObject

p_INDEX = re.compile(r' "([^"]+)" ')
p_SoftBreak = re.compile('\x0b+')


class P(SilvaObject):
    """This implements the Markup type as specified in the schema"""

    # map Word styles to silva styles
    style_map = {
        'Bold': 'strong',
        'Italic': 'em',
        'Underline': 'underline',
        'Superscript': 'super',
        'Subscript': 'sub',
    }
    

    __handled_fields = [wdFieldIndexEntry, wdFieldHyperlink]

    def __init__(self, original, handle_styles=1):
        SilvaObject.__init__(self, original)

        self.text = u''
        self.markup = []
        self._non_indexes = [] 
        self._non_indexes_sorted = True

        self.handleWordParagraph(original, handle_styles)
            

    def handleWordParagraph(self, wp, handle_styles=1):
       
        def merge(list1, list2):
            """merge two insertions lists, preserving sorting""" 
            sort_dict = {}
            
            for index, text in list1+list2:
                sort_dict.setdefault(index, []).append(text)

            indexes = sort_dict.keys()
            indexes.sort()
            
            result = []
            for index in indexes:
                for text in sort_dict[index]:
                    result.append((index, text))

            return result                    

        if wp is None: 
            return

        
        text = wp.Range()
        wp_start = wp.Range.Start
        wp_end = wp.Range.End
        
        if stripControlChars(text.strip()) == '':
            return
        xml = u''
        inserts = []

        if handle_styles:
            inserts = self.processStyles(wp)
            inserts = merge(inserts, self.processIndexes(wp))
            inserts = merge(inserts, self.processLinks(wp))

        self._blank_indexes(wp)
        
        self.setupKitchen()
        inserts = \
            filter(lambda (index, tag): index is not None,
            map(lambda (index, tag): (self.cookIndex(index), tag), 
            map(lambda (index, tag): (index - wp_start, tag), 
            map(lambda (index, tag): 
                (index < wp_start and wp_start or index, tag), 
            map(lambda (index, tag): 
                (index > wp_end and wp_end or index, tag), 
                    inserts)))))

        next = 0
        for index, insertion in inserts:
            xml += replaceMap(text[next:index])
            xml += insertion
            next = index 

        xml += replaceMap(text[next:])
        xml = p_SoftBreak.sub('<br/>', xml)
        xml = stripControlChars(xml)
                
        self.text = xml
          


    def processStyles(self, wp):
        """processes markup like bold and italic"""
       
        styles = []
        styles_dict = {}
        wp_start = wp.Range.Start
        wp_end = wp.Range.End
        
        for word_style, silva_style in self.style_map.items():
            wp_range = wp.Range
            find = wp_range.Find
            self.clearFind(find)
            setattr(find.Font, word_style, True)
            while find.Execute(Replace=wdReplaceNone):
                if wp_range.Start < wp_start: continue
                #fuzz needed for reasons i do not comprehend
                if wp_range.End +1 >= wp_end: break 
                styles_dict.setdefault(wp_range.Start, []).append(
                    (word_style, True))
                styles_dict.setdefault(wp_range.End, []).insert(0, 
                    (word_style, False))
      
        index_list = styles_dict.keys()
        index_list.sort()
        for index in index_list:
            styles += map(lambda (style, onoff): (index, style, onoff),
                 styles_dict[index])
        styles = self.wellform(styles)
        result = []
        
        for start, style, active in styles:
            if active: on_off = ''
            else: on_off = '/'
            tag = '<%s%s>' % (on_off, self.style_map[style], )
            result.append((start, tag))

        return result     


    def processIndexes(self, wp):
            
        if len(wp.Range.Fields) == 0:
            return []
    
        wp_start = wp.Range.Start
        result = []
        for field in wp.Range.Fields:
            if not field.type == wdFieldIndexEntry: continue 
            
            start = field.Code.Start
            end = field.Code.End
            
            self.nonIndex(start, end)
            
            m = p_INDEX.search(field.Code())
            if m is None: continue
            name = m.group(1)
            
            tag = "<index name='%s' />" % (name, )
            result.append((end+1, tag))
   
        return result


    def processLinks(self, wp):

        if len(wp.Range.Hyperlinks) == 0:
            return []
        
        wp_start = wp.Range.Start
        result = []
        
        for link in wp.Range.Hyperlinks:
            
            start = link.Range.start
            end = link.Range.end
            
            url = link.address
            text = link.TextToDisplay

            start_index = start
            end_index = start + len(text) 
            self.nonIndex(end_index, end)
            
            stag = '<link url="%s">' % (url, )
            etag = '</link>'
            
            result.append((start_index, stag))
            result.append((end+1, etag))
            
        return result

    def nonIndex(self, start, end):

        self._non_indexes += range(start, end+1)
        self._non_indexes_sorted = False
   
    def setupKitchen(self):
        """sets up the kitchen for cooking indexes"""
        
        wp_start = self.original.Range.Start
        wp_end = self.original.Range.End
        if not self._non_indexes_sorted:
            self._non_indexes = dict(map(lambda x: (x, 0), 
                self._non_indexes)).keys()
            self._non_indexes.sort()
            self._non_indexes = \
                map(lambda x: x - wp_start, 
                filter(lambda x: wp_start <= x <= wp_end,
                    self._non_indexes))
            self._non_indexes_sorted = True

   
    def cookIndex(self, raw_index):
        nis = self._non_indexes
        corrector = 0
        if raw_index in nis: return None
        for i in range(0, len(nis)):
            if raw_index <= nis[i]:
                break
            else:
                corrector = i + 1
           
        return raw_index - corrector


    def _blank_indexes(self, wp):
        if len(wp.Range.Fields) == 0: 
            return
        for field in wp.Range.Fields:
            if not field.Type in self.__handled_fields:
                self.nonIndex(field.Code.Start, field.Code.End)


    def clearFind(self, find):
        """clear the given find instance"""
        find.ClearFormatting()
        find.ClearAllFuzzyOptions()
        find.Format = 1
        find.Text = u''
        find.MatchCase = 0
        find.MatchWholeWord = 0
        find.MatchWildcards = 0 
        find.MatchSoundsLike = 0
        find.MatchAllWordForms = 0
        find.Wrap = wdWrapNever 
        find.Forward = True

    def wellform(self, styles):
        """wellform the styles
        
            It basicaly handles cases which would lead to:

                <strong>Foo<em>Bar</strong>baz</em>

            By replacing it by:

                <strong>Foo<em>Bar</em></strong><em>baz</em>
        
            It's not done with xml tags but an internal structure though.

            styles -- a list of tuples (start, style, active) whereby
                start -- index of style change in text
                style -- word name of style to change
                active -- True if style is to be activated, False otherwise

            
            returns a list of tuples, same semantics as styles parameter, but 
            wellformed.
        
        """
        
        result = []
        stack = []
        for start, style, active in styles:
            if active:
                if not style in stack:
                    result.append((start, style, active))
                    stack.append(style)
            else:
                if style in stack:
                    deactivated_styles = []
                    while 1:
                        deactivate_style = stack.pop()
                        result.append((start, deactivate_style, active))
                        if deactivate_style == style: break
                        deactivated_styles.append(deactivate_style)
                   
                    deactivated_styles.reverse()
                    for reactivate_style in deactivated_styles:
                        stack.append(reactivate_style)
                        result.append((start, reactivate_style, True))
         
        if stack:
            # This schould NEVER happen
            print "Styles left in stack: %s" % (stack, )
            
        for style in stack:
            result.append((max(map(lambda x: x[0], styles)), style, False))
         
        return result

    def getXML(self):
        if self.text:
            return u'<p>%s</p>' % self.text
        return u''


class Pre(SilvaObject):

    def getXML(self):

        elements = \
            map(stripControlChars, 
            map(lambda x: replaceMap(x),
            self._elements))
        text = '\n'.join(elements)    
            
        xml = u'<pre>%s</pre>' % (text, )
        return xml

    
